<?php
/**
 * File for the AJAX API
 *
 * @package zeus-gpt
 */

// API key class.
require_once ZEUS_GPT_PLUGIN_DIR . '/inc/services/class-zeusgptapikey.php';

// Customizer class.
require_once ZEUS_GPT_PLUGIN_DIR . '/inc/services/class-zeusgptcustomizer.php';

// Rate limiting class.
require_once ZEUS_GPT_PLUGIN_DIR . '/inc/services/class-zeusgptratelimiter.php';

// API Access class.
require_once ZEUS_GPT_PLUGIN_DIR . '/inc/services/class-zeusgptapiaccess.php';

// Chatlog class.
require_once ZEUS_GPT_PLUGIN_DIR . '/inc/services/class-zeusgptchatlog.php';

// Utils class.
require_once ZEUS_GPT_PLUGIN_DIR . '/inc/misc/class-zeusgptutil.php';

// Constants class.
require_once ZEUS_GPT_PLUGIN_DIR . '/inc/misc/class-zeusgptconstants.php';

/**
 * Class responsible for the AJAX API
 */
class ZeusGPTAJAX {

	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		add_action( 'wp_ajax_gfx_zeus_api_key', array( $this, 'api_key_handler' ) );
		add_action( 'wp_ajax_gfx_zeus_chat_prompt', array( $this, 'prompt_handler' ) );
		add_action( 'wp_ajax_nopriv_gfx_zeus_chat_prompt', array( $this, 'prompt_handler' ) );
		add_action( 'wp_ajax_gfx_zeus_settings', array( $this, 'settings_handler' ) );
		add_action( 'wp_ajax_gfx_zeus_get_chatlog', array( $this, 'get_chatlog' ) );
		add_action( 'wp_ajax_gfx_zeus_delete_chatlog', array( $this, 'delete_chatlog' ) );
		add_action( 'wp_ajax_gfx_zeus_delete_all_chatlog', array( $this, 'delete_all_chatlog' ) );
		add_action( 'wp_ajax_gfx_zeus_chatlog_filter', array( $this, 'chatlog_filter' ) );
	}

	/**
	 * Send error
	 *
	 * @param  string $message - error message.
	 * @param  array  $custom_body - body of the response.
	 * @param  number $code - HTTP response code.
	 * @return void
	 */
	private function send_error( $message, $custom_body = null, $code = null ) {

		if ( ! empty( $custom_body ) ) {
			wp_send_json_error( $custom_body, $code );
			wp_die();
		}

		wp_send_json_error(
			array(
				'message' => $message,
			),
			$code
		);
		wp_die();
	}

	/**
	 * Handles the AJAX request for the API key
	 *
	 * @return void
	 */
	public function api_key_handler() {
		if ( empty( $_POST['nonce'] ) ||
		! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'api-input' ) ) {
			$this->send_error( esc_html__( 'There was an error. Please try again.', 'zeus-gpt' ) );
		}

		$key = ! empty( $_POST['api-key'] ) ? sanitize_text_field( wp_unslash( $_POST['api-key'] ) ) : false;

		if ( ! $key ) {
			$this->send_error( esc_html__( 'There was an error. Please try again.', 'zeus-gpt' ) );
		}

		$result = ZeusGPTAPIKey::set_key( $key );

		if ( ! $result ) {
			$this->send_error( esc_html__( 'There was an error in storing the key. Please try again.', 'zeus-gpt' ) );
		}

		wp_send_json_success( array( 'message' => esc_html__( 'Key stored successfully', 'zeus-gpt' ) ), 200 );
		wp_die();
	}

	/**
	 * Handles the AJAX request for prompt
	 *
	 * @return void
	 */
	public function prompt_handler() {
		// nonce check.
		if ( empty( $_POST['nonce'] ) ||
		! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'api-input' ) ) {
			$this->send_error( esc_html__( 'There was an error. Please try again.', 'zeus-gpt' ) );
		}

		// prompt check.
		$prompt = ! empty( $_POST['chat-prompt'] ) ? sanitize_text_field( wp_unslash( $_POST['chat-prompt'] ) ) : false;

		if ( ! is_user_logged_in() && ZeusGPTRateLimiter::is_rate_limited() ) {
			$this->send_error(
				'',
				array(
					'body' => array(
						'message' => esc_html__( 'You are sending too many requests in a short amount of time.', 'zeus-gpt' ),
					),
				),
				429
			);
		}

		// history.
		$history = ! empty( $_POST['chat-history'] ) ? sanitize_text_field( wp_unslash( $_POST['chat-history'] ) ) : false;

		$messages = array();

		if ( ! empty( $history ) ) {
			$messages = json_decode( $history );
		}

		$key = ZeusGPTAPIKey::get_key();

		// check if key exists.
		if ( ! $key ) {
			$this->send_error( esc_html__( 'API Key not found', 'zeus-gpt' ) );
		}

		// behaviour.
		$behaviour = ! empty( $_POST['chat-behaviour'] ) ? sanitize_text_field( wp_unslash( $_POST['chat-behaviour'] ) ) : false;

		if ( $behaviour ) {
			$messages = array_merge(
				array(
					array(
						'role'    => 'system',
						'content' => $behaviour,
					),
				),
				$messages
			);
		}

		$messages = array_merge(
			$messages,
			array(
				array(
					'role'    => 'user',
					'content' => $prompt,
				),
			)
		);
		$data     = array(
			'model'    => 'gpt-3.5-turbo',
			'messages' => $messages,
		);

		// time for the chatlog.
		$user_time = ZeusGPTUtil::get_date( 'now', ZeusGPTConstants::$db_date_format );

		$response = ZeusGPTAPIAccess::request( $data, $key );

		if ( is_wp_error( $response ) || wp_remote_retrieve_response_code( $response ) !== 200 ) {

			$message = esc_html__( 'There was an issue in communicating with the API.', 'zeus-gpt' );

			$this->send_error(
				'',
				array(
					'body' => array(
						'message' => $message,
					),
				)
			);
		}

		$response_message = $this->extract_message( $response );

		// add to chatlog.
		if ( ! is_user_logged_in() && ! empty( $response_message ) ) {
			ZeusGPTChatLog::update_chatlog(
				$prompt,
				$user_time,
				$response_message,
				ZeusGPTUtil::get_date( 'now', ZeusGPTConstants::$db_date_format )
			);
		}

		wp_send_json_success( $response, 200 );
		wp_die();
	}

	/**
	 * Extracts message from response
	 *
	 * @param  array $response - response array.
	 * @return string
	 */
	private function extract_message( $response ) {
		if ( ! array_key_exists( 'body', $response ) ) {
			return false;
		}

		$data = null;

		try {
			$data = json_decode( $response['body'], true );
		} catch ( Exception $e ) {
			return false;
		}

		// if data is empty, then return.
		if ( null === $data ) {
			return false;
		}

		if (
			array_key_exists( 'choices', $data ) &&
			count( $data['choices'] ) > 0 &&
			array_key_exists( 'message', $data['choices'][0] ) &&
			array_key_exists( 'content', $data['choices'][0]['message'] )
		) {
			return $data['choices'][0]['message']['content'];
		} else {
			return false;
		}
	}

	/**
	 * Handles the AJAX request for customizations.
	 *
	 * @return void
	 */
	public function settings_handler() {
		// nonce check.
		if ( empty( $_POST['nonce'] ) ||
		! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'api-input' ) ) {
			$this->send_error( esc_html__( 'There was an error. Please try again.', 'zeus-gpt' ) );
		}

		// customizer.
		$customizer_result = ZeusGPTCustomizer::handle_input();

		// API key.
		$key = ! empty( $_POST['api-key'] ) ? sanitize_text_field( wp_unslash( $_POST['api-key'] ) ) : false;

		if ( ! $key ) {
			$this->send_error( esc_html__( 'API key not provided.', 'zeus-gpt' ) );
		}

		$key_result = ZeusGPTAPIKey::set_key( $key );

		// rate limiting.
		$rate_limiting = ! empty( $_POST['rate-limiting'] ) ? sanitize_text_field( wp_unslash( $_POST['rate-limiting'] ) ) : false;

		if ( ! $rate_limiting ) {
			$this->send_error( esc_html__( 'Rate limiting not provided.', 'zeus-gpt' ) );
		}

		$rate_limiting_result = ZeusGPTRateLimiter::set_rate_limit( $rate_limiting );

		// chatlog toggle.
		$chatlog_toggle = ! empty( $_POST['chatlog-toggle'] ) ? sanitize_text_field( wp_unslash( $_POST['chatlog-toggle'] ) ) : false;

		if ( ! $chatlog_toggle ) {
			$this->send_error( esc_html__( 'Chatlog Toggle switch not found!', 'zeus-gpt' ) );
		}

		$chatlog_toggle_result = ZeusGPTChatLog::set_toggle_option( 'true' === $chatlog_toggle );

		// chatlog popup delete toggle.
		$chatlog_delete_toggle = ! empty( $_POST['chatlog-delete-dialog-toggle'] ) ? sanitize_text_field( wp_unslash( $_POST['chatlog-delete-dialog-toggle'] ) ) : false;

		if ( ! $chatlog_delete_toggle ) {
			$this->send_error( esc_html__( 'Chatlog Popup Toggle switch not found!', 'zeus-gpt' ) );
		}

		$chatlog_delete_toggle_result = ZeusGPTChatLog::set_delete_toggle_option( 'true' === $chatlog_delete_toggle );

		wp_send_json_success( array( 'message' => esc_html__( 'Settings were saved successfully.', 'zeus-gpt' ) ), '200' );
		wp_die();
	}

	/**
	 * Retreives the chatlog
	 *
	 * @return void
	 */
	public function get_chatlog() {
		// nonce check.
		if ( empty( $_POST['nonce'] ) ||
		! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'api-input' ) ) {
			$this->send_error( esc_html__( 'There was an error. Please try again.', 'zeus-gpt' ) );
		}

		$ip = ! empty( $_POST['ip'] ) ? sanitize_text_field( wp_unslash( $_POST['ip'] ) ) : false;

		$log = ZeusGPTChatLog::get_chatlog( $ip );

		if ( null === $log ) {
			$this->send_error( esc_html__( 'Chatlog not found!', 'zeus-gpt' ) );
		}

		wp_send_json_success( $log, '200' );
		wp_die();
	}

	/**
	 * Deletes the chatlog
	 *
	 * @return void
	 */
	public function delete_chatlog() {
		// nonce check.
		if ( empty( $_POST['nonce'] ) ||
		! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'api-input' ) ) {
			$this->send_error( esc_html__( 'There was an error. Please try again.', 'zeus-gpt' ) );
		}

		$ip = ! empty( $_POST['ip'] ) ? sanitize_text_field( wp_unslash( $_POST['ip'] ) ) : false;

		$response = ZeusGPTChatLog::delete_chatlog( $ip );

		if ( ! $response ) {
			$this->send_error( esc_html__( 'Chatlog could not be deleted!', 'zeus-gpt' ) );
		}

		wp_send_json_success(
			array(
				'message' => esc_html__( 'Chatlog deleted successfully!', 'zeus-gpt' ),
			),
			'200'
		);
		wp_die();
	}

	/**
	 * Deletes all chatlogs
	 *
	 * @return void
	 */
	public function delete_all_chatlog() {
		// nonce check.
		if ( empty( $_POST['nonce'] ) ||
		! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'api-input' ) ) {
			$this->send_error( esc_html__( 'There was an error. Please try again.', 'zeus-gpt' ) );
		}

		$response = ZeusGPTChatLog::delete_all_chatlog();

		if ( ! $response ) {
			$this->send_error( esc_html__( 'Chatlogs could not be deleted!', 'zeus-gpt' ) );
		}

		wp_send_json_success(
			array(
				'message' => esc_html__( 'Chatlogs deleted successfully!', 'zeus-gpt' ),
			),
			'200'
		);
		wp_die();
	}

	/**
	 * Filters the chatlog
	 *
	 * @return void
	 */
	public function chatlog_filter() {
		// nonce check.
		if ( empty( $_POST['nonce'] ) ||
		! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'api-input' ) ) {
			ZeusGPTChatLog::the_chatlog_empty_element( esc_html__( 'An error occurred. Please try again.', 'zeus-gpt' ), 'chatlog-empty-error' );
			wp_die();
		}

		$filter = ! empty( $_POST['filter'] ) ? sanitize_text_field( wp_unslash( $_POST['filter'] ) ) : false;

		if ( empty( $filter ) ) {
			ZeusGPTChatLog::the_chatlog_empty_element( esc_html__( 'Filter not found. Please try again.', 'zeus-gpt' ), 'chatlog-empty-error' );
			wp_die();
		}

		$result = ZeusGPTChatLog::chatlog_filter( $filter );

		if ( ! $result ) {
			ZeusGPTChatLog::the_chatlog_empty_element( null, 'chatlog-empty-error' );
			wp_die();
		}

		foreach ( $result as $data ) {
			ZeusGPTChatLog::the_chatlog_box( $data );
		}

		wp_die();

	}

}
