<?php
/**
 * File for the Rate Limiter class
 *
 * @package zeus-gpt
 */

/**
 * Class responsible for the rate limiting
 */
class ZeusGPTRateLimiter {


	/**
	 * Responsible for registering hooks/functionality.
	 *
	 * @return void
	 */
	public function __construct() {
		add_action( 'zeusgpt_rate_limiting_event', array( $this, 'reduce_rate_limiter' ) );
		add_filter( 'cron_schedules', array( $this, 'cron_schedules' ) ); // phpcs:ignore
	}

	/**
	 * Sets the rate limit option
	 *
	 * @param  string $limit - number of requests to be limited.
	 * @return boolean
	 */
	public static function set_rate_limit( $limit ) {
		$limit = intval( $limit );
		// check if the option exists.
		if ( get_option( 'gfx_zeus_rate_limit' ) ) {
			return update_option( 'gfx_zeus_rate_limit', $limit, false );
		} else {
			return add_option( 'gfx_zeus_rate_limit', $limit, '', false );
		}
	}

	/**
	 * Gets the rate limit option
	 *
	 * @param  number $default - default value to be returned if nothing is found.
	 * @return number
	 */
	public static function get_rate_limit( $default = 10 ) {
		return intval( get_option( 'gfx_zeus_rate_limit', $default ) );
	}

	/**
	 * Checks if the current address is rate limited
	 *
	 * @return void
	 */
	public static function is_rate_limited() {
		// 1. get the ip address of the current request.
		$ip_address      = null;
		$limit           = self::get_rate_limit();
		$is_rate_limited = false;

		if ( ! empty( $_SERVER['REMOTE_ADDR'] ) ) {
			$ip_address = sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) );
		} else {
			return;
		}

		// 2. get the rate limiting option from the options table.
		$rate_limiting        = get_option( 'gfx_zeus_rate_limiting' );
		$rate_limiting_option = $rate_limiting;

		if ( ! is_array( $rate_limiting ) ) {
			$rate_limiting = array();
		}

		// 3. check if the ip address exists in the rate limiting option.
		if ( array_key_exists( $ip_address, $rate_limiting ) ) {
			// 5. if the ip address exists then update it.
			$contents = $rate_limiting[ $ip_address ];

			$count     = $contents['count'];
			$last_used = $contents['last_used'];

			// if count is greater than limit, then don't process.
			if ( $contents['count'] >= $limit ) {
				$is_rate_limited = true;
			} else {
				// update.
				$contents['count']++;
				$contents['last_used']        = time();
				$rate_limiting[ $ip_address ] = $contents;
			}
		} else {
			// 4. if the ip address does not exist then add it.
			$rate_limiting[ $ip_address ] = array(
				'count'     => 1,
				'last_used' => time(),
			);
		}

		// 6. Update the options table with the new array.
		if ( is_array( $rate_limiting_option ) ) {
			update_option( 'gfx_zeus_rate_limiting', $rate_limiting, false );
		} else {
			add_option( 'gfx_zeus_rate_limiting', $rate_limiting, '', false );
		}

		return $is_rate_limited;
	}

	/**
	 * Responsible for decreasing the count.
	 *
	 * @return void
	 */
	public function reduce_rate_limiter() {
		$rate_limiting = get_option( 'gfx_zeus_rate_limiting' );

		if ( empty( $rate_limiting ) ) {
			return;
		}

		foreach ( $rate_limiting as $ip => $data ) {
			$time_exceeded = time() - $data['last_used'] >= 60;
			if ( $time_exceeded && $data['count'] > 0 ) {
				$data['count']        = $data['count'] - 1;
				$rate_limiting[ $ip ] = $data;
			} elseif ( $time_exceeded && 0 === $data['count'] ) {
				unset( $rate_limiting[ $ip ] );
			}
		}

		update_option( 'gfx_zeus_rate_limiting', $rate_limiting, false );

	}


	/**
	 * Add one minute schedule
	 *
	 * @param  mixed $schedules - array of schedules.
	 * @return array
	 */
	public function cron_schedules( $schedules ) {
		if ( isset( $schedules['zeusgpt-1-min'] ) ) {
			return $schedules;
		}

		$schedules['zeusgpt-1-min'] = array(
			'interval' => 60,
			'display'  => esc_html__( 'Once every 1 minutes', 'zeus-gpt' ),
		);

		return $schedules;
	}

	/**
	 * Handles the activation for this class.
	 *
	 * @return void
	 */
	public static function activation() {
		if ( ! wp_next_scheduled( 'zeusgpt_rate_limiting_event' ) ) {
			wp_schedule_event( time(), 'zeusgpt-1-min', 'zeusgpt_rate_limiting_event' );
		}
	}

	/**
	 * Handles the deactivation for this class.
	 *
	 * @return void
	 */
	public static function deactivation() {
		wp_clear_scheduled_hook( 'zeus_get_rate_limiting_event' );
	}


}
