import markdown from './drawdown';

// Creates a chat element.
export const createChatElement = (content, className = '', wrapperClassName = '', isMarkdown = false) => {
    const wrapper = document.createElement('div');
    wrapper.className = 'chat-element-wrapper ' + wrapperClassName;

    const div = document.createElement('div');
    div.className = 'chat-element ' + className;
    
    if ( isMarkdown ) {
        div.innerHTML = markdown(content);
    } else {
        div.textContent = content;
    }
    
    wrapper.appendChild(div);
    return wrapper;
};

// Creates a chat error.
const createChatError = (message, prompt, className = '') => {
    const wrapper = document.createElement('div');
    wrapper.className = 'chat-error ' + className;

    const errorIcon = document.createElement('div');
    errorIcon.className = 'error-icon';
    errorIcon.setAttribute('tooltip', message);
    errorIcon.setAttribute('flow', 'left');
    errorIcon.innerHTML =
        '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24"><path d="M11,15H13V17H11V15M11,7H13V13H11V7M12,2C6.47,2 2,6.5 2,12A10,10 0 0,0 12,22A10,10 0 0,0 22,12A10,10 0 0,0 12,2M12,20A8,8 0 0,1 4,12A8,8 0 0,1 12,4A8,8 0 0,1 20,12A8,8 0 0,1 12,20Z" /></svg>';

    wrapper.appendChild(errorIcon);

    return wrapper;
};

const getChatHistory = (container) => {
    const historyEls = container.querySelectorAll(
        '.chat-element-wrapper:nth-last-child(-n+10):not(.chat-error-wrapper)'
    );

    const history = [];

    if (historyEls.length > 0) {
        historyEls.forEach((message) => {
            let type = 'assistant';

            if (message.classList.contains('chat-element-wrapper--personal')) {
                type = 'user';
            }

            const el = message.querySelector('.chat-element');

            if (el == null) return;

            history.push({ role: type, content: el.textContent });
        });
    }

    return history;
};

const createChatLoadingElement = () => {
    const wrapper = document.createElement('div');
    wrapper.className = 'chat-element-wrapper loading-wrapper';

    const div = document.createElement('div');
    div.className = 'chat-element';

    div.innerHTML =
        '<div class="chat-loading-icon"></div><div class="chat-loading-icon"></div><div class="chat-loading-icon"></div>';

    wrapper.appendChild(div);
    return wrapper;
};

const addToChatLoading = (element, message) => {
    const inner = element.querySelector('.chat-element');
    inner.innerHTML = markdown(message);
    element.classList.remove('loading-wrapper');
    return element;
};

const createFormData = (prompt, behaviour, history) => {
    const formData = new FormData();
    formData.append('chat-prompt', prompt);
    formData.append('chat-history', JSON.stringify(history));
    formData.append('action', 'gfx_zeus_chat_prompt');
    formData.append('nonce', GFXZEUSOBJPUBLIC['nonce']);

    if (behaviour != null) {
        formData.append('chat-behaviour', behaviour);
    }

    return formData;
};

// Sends message to the server
const handlePromptInput = async (e) => {
    e.preventDefault();

    const parent = e.target.closest('.gfx-zeus-chat-container');

    if (parent == null) return;

    const emptyEl = parent.querySelector('.gfx-zeus-chat-empty');

    const chatContainer = parent.querySelector('.gfx-zeus-chat-conversation');

    if (chatContainer == null) return;

    const promptEl = parent.querySelector('.gfx-zeus-chat-prompt');

    if (promptEl == null) return;

    // create form data.
    const prompt = promptEl.innerText.trim(); // trim() removes whitespace from both sides of the string.

    // don't process if the prompt is empty.
    if (prompt == '') {
        promptEl.textContent = '';
        return;
    }

    if (emptyEl != null) {
        emptyEl.classList.add('empty-hide');
    }

    // empty the chat prompt
    promptEl.textContent = '';

    // gather history messages
    const history = getChatHistory(chatContainer);

    // system behaviour
    let behaviour = null;
    const behaviourEl = parent.querySelector('input[name="chat-behaviour"]');
    if (behaviourEl != null) behaviour = behaviourEl.value;

    // create form data
    const formData = createFormData(prompt, behaviour, history);

    // add the message to the chat.
    const personalElement = createChatElement(
        prompt,
        'chat-element--personal',
        'chat-element-wrapper--personal'
    );

    // add personal message to the chat.
    chatContainer.appendChild(personalElement);
    chatContainer.scrollTo(0, chatContainer.scrollHeight);

    // create loading element
    const chatLoadingElement = createChatLoadingElement();
    setTimeout(() => {
        chatContainer.appendChild(chatLoadingElement);
        chatContainer.scrollTo(0, chatContainer.scrollHeight);
    }, 1000);

    // send request
    const response = await fetch(GFXZEUSOBJPUBLIC['ajax-url'], {
        method: 'POST',
        body: formData,
    });

    const resJSON = await response.json();

    let bodyJSON = null;

    if (typeof resJSON.data.body === 'string') {
        bodyJSON = JSON.parse(resJSON.data.body);
    } else {
        bodyJSON = resJSON.data.body;
    }

    if (!resJSON.success || typeof bodyJSON.choices === 'undefined') {
        personalElement.appendChild(createChatError(bodyJSON.message, prompt));
        personalElement.classList.add('chat-error-wrapper');

        setTimeout(() => {
            if (chatLoadingElement == null) return;
            chatLoadingElement.remove();
        }, 1000);

        return;
    }

    const message = bodyJSON.choices[0].message.content;

    addToChatLoading(chatLoadingElement, message);

    chatContainer.scrollTo(0, chatContainer.scrollHeight);
};

// handles the reload for a failed message.
export const handleReload = (element) => {
    if (typeof element.prompt === 'undefined') return;
    if (element.prompt === '') return;
};

// exports the service function
export default () => {
    const chatFormEls = document.querySelectorAll(
        '.gfx-zeus-chat-container .gfx-zeus-chat-form'
    );
    if (chatFormEls.length < 1) return;
    chatFormEls.forEach((chatForm) => {
        // form submit event listener
        chatForm.addEventListener('submit', handlePromptInput);
        // enter key event listener
        const promptEl = chatForm.querySelector('.gfx-zeus-chat-prompt');
        if (promptEl == null) return;
        promptEl.addEventListener('keydown', (e) => {
            if (e.keyCode == 13 && !e.shiftKey) {
                e.preventDefault();
                chatForm.requestSubmit();
            }
        });
    });
};
