const { BlockControls } = wp.blockEditor;
const {
    ToolbarGroup,
    ToolbarDropdownMenu,
    ButtonGroup,
    Button,
    Modal,
    TextareaControl,
} = wp.components;
const { __ } = wp.i18n;
const { addFilter } = wp.hooks;
const { Fragment, useState } = wp.element;
import { MAIN_ICON, LOADING_BAR } from './icons.js';

const allowedBlockTypes = [
    'core/preformatted',
    'core/paragraph',
    'core/list-item',
    'core/code',
    'core/freeform',
    'core/heading',
    'core/list',
];

const toolbarOptions = [
    {
        id: 'write_paragraph',
        title: __('Write a paragraph about this', 'zeus-gpt'),
        prompt: 'Rewrite the following content:\n\n\n\n [CONTENT]',
        blockType: 'core/paragraph',
    },
    {
        id: 'correct_grammatical_mistakes',
        title: __('Correct grammatical mistakes', 'zeus-gpt'),
        prompt: 'Correct Grammatical mistakes in the following content:\n\n\n\n [CONTENT]',
        blockType: 'core/paragraph',
    },
    {
        id: 'summarize_paragraph',
        title: __('Summarize', 'zeus-gpt'),
        prompt: 'Summarize the following content in a paragraph:\n\n\n\n [CONTENT]',
        blockType: 'core/paragraph',
    },
    {
        id: 'rewrite_in_formal_words',
        title: __('Rewrite formally', 'zeus-gpt'),
        prompt: 'Rewrite the following content in formal words:\n\n\n\n [CONTENT]',
        blockType: 'core/paragraph',
    },
    {
        id: 'summarize_in_bullet_points',
        title: __('Summarize in bullet points', 'zeus-gpt'),
        prompt: 'Summarize the following content in bullet points:\n\n\n\n [CONTENT]',
        blockType: 'core/paragraph',
    },
    {
        id: 'generate_code',
        title: __('Generate code', 'zeus-gpt'),
        prompt: 'Generate code about the following content:\n\n\n\n [CONTENT]',
        blockType: 'core/code',
    },
    {
        id: 'custom_prompt',
        title: __('Custom prompt', 'zeus-gpt'),
        blockType: 'core/paragraph',
    },
];

const getSelectedBlockIds = () => {
    const editor = wp.data.select('core/block-editor');
    let blockIds = editor.getMultiSelectedBlockClientIds();

    if (blockIds.length === 0) {
        blockIds = [editor.getSelectedBlockClientId()];
    }

    return blockIds;
};

const getSelection = (blockIds) => {
    const editor = wp.data.select('core/block-editor');

    let start = editor.getSelectionStart();
    let end = editor.getSelectionEnd();

    // return the start and end if it is the same block
    if (start.clientId === end.clientId) {
        return [start, end];
    }

    // reverse the points if there are more than one blocks
    if (blockIds.length > 0 && blockIds[0] === end.clientId) {
        [start, end] = [end, start];
    }

    return [start, end];
};

const updateBlock = (block, content) => {
    const editor = wp.data.dispatch('core/block-editor');

    if (!('choices' in content)) {
        editor.removeBlocks(block.clientId);
        return;
    }

    const attributes = block.attributes;
    attributes.content = content.choices[0].message.content;

    attributes.content = attributes.content.replace(/\n/g, '<br>');

    editor.updateBlock(block.clientId, attributes);
};

const sendRequest = async (prompt) => {
    const formData = new FormData();
    formData.append('chat-prompt', prompt);
    formData.append('action', 'gfx_zeus_chat_prompt');
    formData.append('nonce', GFXZEUSOBJPUBLIC['nonce']);

    const response = await fetch(GFXZEUSOBJPUBLIC['ajax-url'], {
        method: 'POST',
        body: formData,
    });

    const resJSON = await response.json();

    let bodyJSON = null;

    if (typeof resJSON.data.body === 'string') {
        bodyJSON = JSON.parse(resJSON.data.body);
    } else {
        bodyJSON = resJSON.data.body;
    }

    return bodyJSON;
};

function extractContent(block) {
    const attributes = ['content', 'citation', 'value', 'values', 'text'];
    let result = null;
    attributes.forEach((attribute) => {
        if (attribute in block.attributes) {
            result = block.attributes[attribute];
            return false;
        }
    });

    return result;
}

const getSelectedContent = (blockIds, start, end) => {
    const editor = wp.data.select('core/block-editor');

    let result = '';

    blockIds.forEach((blockId) => {
        const block = editor.getBlock(blockId);
        let content = extractContent(block);
        // content = wp.richText.create( { html: content } ).text;

        let trimStart = 0;
        let trimEnd = content.length;

        if (start.clientId === blockId && 'offset' in start) {
            trimStart = start.offset;
        }

        if (end.clientId === blockId && 'offset' in end) {
            trimEnd = end.offset;
        }

        let trimmedContent = content;

        if (trimStart !== trimEnd) {
            trimmedContent = content.substring(trimStart, trimEnd);
        }

        result += '\n' + trimmedContent;

        if (block.innerBlocks.length > 0) {
            result += getSelectedContent(
                block.innerBlocks.map((block) => block.clientId),
                start,
                end
            );
        }
    });

    return result.trim();
};

const createBlock = async (type = 'core/paragraph') => {
    const editor = wp.data.select('core/block-editor');

    // get all the selected block ids.
    const blockIds = getSelectedBlockIds();
    // get the selected start and end blocks.
    const [start, end] = getSelection(blockIds);
    // get the last block
    let lastBlock = editor.getBlock(end.clientId);

    // create block
    let createdBlock = wp.blocks.createBlock(type, {
        content: LOADING_BAR,
    });

    // get index of first block
    let index = editor.getBlockIndex(lastBlock.clientId) + 1;
    // get parent client id of first block
    let parentBlockId = editor.getBlockRootClientId(start.clientId);

    // insert autocomplete block before the selected block
    await wp.data
        .dispatch('core/block-editor')
        .insertBlock(createdBlock, index, parentBlockId);

    return createdBlock;
};

const customPromptHandler = async (prompt) => {
    const blockIds = getSelectedBlockIds();
    const [start, end] = getSelection(blockIds);

    const block = await createBlock();
    const content = getSelectedContent(blockIds, start, end);

    prompt += '\n\n\n ' + content;

    const response = await sendRequest(prompt);
    updateBlock(block, response);
};

const clickHandler = async (option, openModal) => {
    if (option.id === 'custom_prompt') {
        openModal();
        return;
    }

    const blockIds = getSelectedBlockIds();
    const [start, end] = getSelection(blockIds);

    const block = await createBlock(option.blockType);
    const content = getSelectedContent(blockIds, start, end);

    const prompt = option.prompt.replace('[CONTENT]', content);

    const response = await sendRequest(prompt);
    updateBlock(block, response);
};

const toolbarRenderer = (BlockEdit) => {
    return (props) => {
        const [isOpen, setOpen] = useState(false);
        const openModal = () => setOpen(true);
        const closeModal = () => setOpen(false);

        const [customPrompt, setCustomPrompt] = useState('');

        if (!allowedBlockTypes.includes(props.name)) {
            return <BlockEdit {...props} />;
        }

        const individualDropdownItem = (option) => {
            const result = {
                title: option.title,
                onClick: () => clickHandler(option, openModal),
            };

            return result;
        };

        return (
            <Fragment>
                <BlockEdit {...props} />
                <BlockControls group="block">
                    <ToolbarGroup>
                        <ToolbarDropdownMenu
                            icon={MAIN_ICON}
                            label={__(
                                'Select how do you want AI to edit your content',
                                'zeus-gpt'
                            )}
                            controls={toolbarOptions.map(
                                individualDropdownItem
                            )}
                        />
                    </ToolbarGroup>
                </BlockControls>
                {isOpen && (
                    <Modal
                        title={__('ZeusGPT Custom Prompt', 'zeus-gpt')}
                        onRequestClose={closeModal}
                    >
                        <TextareaControl
                            label={__('Prompt', 'zeus-gpt')}
                            value={customPrompt}
                            onChange={(value) => setCustomPrompt(value)}
                        />

                        <ButtonGroup>
                            <Button
                                variant="primary"
                                onClick={async () => {
                                    closeModal();
                                    await customPromptHandler(customPrompt);
                                }}
                                style={{ margin: '0 5px' }}
                            >
                                {__('Continue', 'zeus-gpt')}
                            </Button>
                            <Button
                                variant="secondary"
                                onClick={closeModal}
                                style={{ margin: '0 5px' }}
                            >
                                {__('Cancel', 'zeus-gpt')}
                            </Button>
                        </ButtonGroup>
                    </Modal>
                )}
            </Fragment>
        );
    };
};

addFilter('editor.BlockEdit', 'zeusgpt/toolbar', toolbarRenderer);
